import os
import csv
import requests
from bs4 import BeautifulSoup
import time

# Файлы
links_file = 'istmat_links.csv'
meta_file = 'meta.csv'
log_file = 'meta_logs.txt'

# Колонки для метаданных
FIELDS = [
    "id", "name", "link", "author", "stream", "genre",
    "state", "period", "source", "file_link"
]

def read_last_index():
    """Чтение индекса последней успешно обработанной строки из лога."""
    if not os.path.exists(log_file):
        return 0
    with open(log_file, 'r', encoding='utf-8') as f:
        lines = f.readlines()
        for line in reversed(lines):
            if line.startswith("SUCCESS"):
                return int(line.split()[1]) + 1
    return 0

def log_event(event, idx, url, msg=""):
    """Запись события в лог."""
    with open(log_file, 'a', encoding='utf-8') as f:
        f.write(f"{event} {idx} {url} {msg}\n")

def parse_requisites(soup):
    """Парсинг таблицы 'Реквизиты'."""
    meta = {
        "author": "", "stream": "", "genre": "",
        "state": "", "period": "", "source": ""
    }
    field_map = {
        "Автор": "author",
        "Направление": "stream",
        "Жанр": "genre",
        "Государство": "state",
        "Период": "period",
        "Источник": "source"
    }
    for section in soup.find_all('div', class_="section"):
        table = section.find("table")
        if table:
            for row in table.find_all("tr"):
                cols = row.find_all("td")
                if len(cols) == 2:
                    label = cols[0].get_text(strip=True)
                    value = cols[1].get_text(strip=True)
                    key = field_map.get(label)
                    if key:
                        meta[key] = value
            break
    return meta

def parse_file_link(soup):
    """Поиск первой ссылки на PDF-файл."""
    for a in soup.find_all("a", href=True):
        if a["href"].endswith(".pdf") and "/files/uploads/" in a["href"]:
            return a["href"]
    return ""

def main():
    # Узнаём, с какой строки начинать (после последнего успешного)
    start_idx = read_last_index()
    with open(links_file, newline='', encoding='utf-8') as infile, \
         open(meta_file, 'a', newline='', encoding='utf-8') as metafile:
        reader = csv.DictReader(infile)
        # Если файл пустой — пишем заголовок
        if os.stat(meta_file).st_size == 0:
            writer = csv.DictWriter(metafile, fieldnames=FIELDS)
            writer.writeheader()
        else:
            writer = csv.DictWriter(metafile, fieldnames=FIELDS)
        for idx, row in enumerate(reader):
            if idx < start_idx:
                continue
            url = row.get("link") or row.get("url") or next(iter(row.values()))
            if not url or not url.startswith("http"):
                log_event("SKIP", idx, url, "not a valid url")
                continue
            try:
                resp = requests.get(url, timeout=15)
                soup = BeautifulSoup(resp.content, 'html.parser')
                name_tag = soup.find("h1", {"id": "page-title"})
                name = name_tag.get_text(strip=True) if name_tag else ""
                meta = parse_requisites(soup)
                file_link = parse_file_link(soup)
                data = {
                    "id": url.split("/")[-1],
                    "name": name,
                    "link": url,
                    "author": meta["author"],
                    "stream": meta["stream"],
                    "genre": meta["genre"],
                    "state": meta["state"],
                    "period": meta["period"],
                    "source": meta["source"],
                    "file_link": file_link
                }
                writer.writerow(data)
                metafile.flush()
                log_event("SUCCESS", idx, url)
                time.sleep(1)  # задержка для бережной работы с сайтом
            except Exception as e:
                log_event("ERROR", idx, url, str(e))
                break  # Остановиться, чтобы при следующем запуске продолжить с этой строки

if __name__ == '__main__':
    main()
